@@ -19,7 +19,7 @@ from __future__ import division
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = '0.10.1'
+__version__ = '0.10.7'
 
 # Used with pypi checks and other messages related to pip
 _wandb_module = 'wandb'
@@ -29,7 +29,7 @@ import sys
 from wandb.errors import Error
 
 # This needs to be early as other modules call it.
-from wandb.errors.term import termlog, termerror, termwarn
+from wandb.errors.term import termsetup, termlog, termerror, termwarn
 
 PY3 = sys.version_info.major == 3 and sys.version_info.minor >= 6
 if PY3:
@@ -43,6 +43,8 @@ init = wandb_sdk.init
 setup = wandb_sdk.setup
 save = wandb_sdk.save
 watch = wandb_sdk.watch
+finish = wandb_sdk.finish
+join = finish
 login = wandb_sdk.login
 helper = wandb_sdk.helper
 Artifact = wandb_sdk.Artifact
@@ -73,11 +75,11 @@ from wandb.wandb_agent import agent
 from wandb.wandb_controller import sweep, controller
 
 from wandb import superagent
-from wandb.jupyteragent import jupyteragent as _secretagent
 
 # from wandb.core import *
 from wandb.viz import visualize
-from wandb import plots
+from wandb import plot
+from wandb import plots  # deprecating this
 from wandb.integration.sagemaker import sagemaker_auth
 
 
@@ -112,12 +114,6 @@ summary = _preinit.PreInitObject("wandb.summary")
 log = _preinit.PreInitCallable(
     "wandb.log", wandb_sdk.wandb_run.Run.log
 )
-join = _preinit.PreInitCallable(
-    "wandb.join", wandb_sdk.wandb_run.Run.join
-)
-finish = _preinit.PreInitCallable(
-    "wandb.finish", wandb_sdk.wandb_run.Run.finish
-)
 save = _preinit.PreInitCallable(
     "wandb.save", wandb_sdk.wandb_run.Run.save
 )
@@ -130,6 +126,10 @@ use_artifact = _preinit.PreInitCallable(
 log_artifact = _preinit.PreInitCallable(
     "wandb.log_artifact", wandb_sdk.wandb_run.Run.log_artifact
 )
+plot_table = _preinit.PreInitCallable(
+    "wandb.plot_table", wandb_sdk.wandb_run.Run.plot_table
+)
+
 # record of patched libraries
 patched = {"tensorboard": [], "keras": [], "gym": []}
 
@@ -144,7 +144,7 @@ lightgbm = _lazyloader.LazyLoader(
 )
 docker = _lazyloader.LazyLoader("wandb.docker", globals(), "wandb.docker")
 jupyter = _lazyloader.LazyLoader("wandb.jupyter", globals(), "wandb.jupyter")
-
+sacred = _lazyloader.LazyLoader("wandb.sacred", globals(), "wandb.integration.sacred")
 
 def ensure_configured():
     global api
