@@ -5,6 +5,7 @@ import copy
 import datetime
 from functools import wraps
 import getpass
+import json
 import logging
 import os
 import shutil
@@ -27,6 +28,7 @@ from wandb import env, util
 from wandb import Error
 from wandb import wandb_agent
 from wandb import wandb_controller
+from wandb import wandb_sdk
 from wandb.apis import InternalApi, PublicApi
 from wandb.integration.magic import magic_install
 from wandb.old.settings import Settings
@@ -84,9 +86,12 @@ def display_error(func):
 _api = None  # caching api instance allows patching from unit tests
 
 
-def _get_cling_api():
+def _get_cling_api(reset=None):
     """Get a reference to the internal api with cling settings."""
     global _api
+    if reset:
+        _api = None
+        wandb_sdk.wandb_setup._setup(_reset=True)
     if _api is None:
         # TODO(jhr): make a settings object that is better for non runs.
         wandb.setup(settings=wandb.Settings(_cli_only_mode=True))
@@ -193,22 +198,31 @@ def projects(entity, display=True):
 def login(key, host, cloud, relogin, anonymously, no_offline=False):
     # TODO: handle no_offline
     anon_mode = "must" if anonymously else "never"
-    wandb.setup(
-        settings=wandb.Settings(_cli_only_mode=True, anonymous=anon_mode, base_url=host)
-    )
-    api = _get_cling_api()
 
+    if host and not host.startswith("http"):
+        raise ClickException("host must start with http(s)://")
+
+    _api = InternalApi()
     if host == "https://api.wandb.ai" or (host is None and cloud):
-        api.clear_setting("base_url", globally=True, persist=True)
+        _api.clear_setting("base_url", globally=True, persist=True)
         # To avoid writing an empty local settings file, we only clear if it exists
         if os.path.exists(Settings._local_path()):
-            api.clear_setting("base_url", persist=True)
+            _api.clear_setting("base_url", persist=True)
     elif host:
-        if not host.startswith("http"):
-            raise ClickException("host must start with http(s)://")
-        api.set_setting("base_url", host.strip("/"), globally=True, persist=True)
+        # force relogin if host is specified
+        _api.set_setting("base_url", host.strip("/"), globally=True, persist=True)
     key = key[0] if len(key) > 0 else None
+    if host or cloud or key:
+        relogin = True
 
+    wandb.setup(
+        settings=wandb.Settings(
+            _cli_only_mode=True,
+            _disable_viewer=relogin,
+            anonymous=anon_mode,
+            base_url=host,
+        )
+    )
     wandb.login(relogin=relogin, key=key, anonymous=anon_mode, host=host, force=True)
 
 
@@ -279,9 +293,10 @@ def init(ctx, project, entity, reset):
         click.echo(
             click.style("Let's setup this directory for W&B!", fg="green", bold=True)
         )
-    api = InternalApi()
+    api = _get_cling_api()
     if api.api_key is None:
         ctx.invoke(login)
+        api = _get_cling_api(reset=True)
 
     viewer = api.viewer()
 
@@ -296,6 +311,7 @@ def init(ctx, project, entity, reset):
             )
         )
         ctx.invoke(login)
+        api = _get_cling_api(reset=True)
 
     # This shouldn't happen.
     viewer = api.viewer()
@@ -379,19 +395,19 @@ def init(ctx, project, entity, reset):
 @click.option(
     "--include-online/--no-include-online",
     is_flag=True,
-    default=False,
+    default=None,
     help="Include online runs",
 )
 @click.option(
     "--include-offline/--no-include-offline",
     is_flag=True,
-    default=True,
+    default=None,
     help="Include offline runs",
 )
 @click.option(
     "--include-synced/--no-include-synced",
     is_flag=True,
-    default=False,
+    default=None,
     help="Include synced runs",
 )
 @click.option(
@@ -438,10 +454,11 @@ def sync(
     clean_old_hours=24,
     clean_force=None,
 ):
-    api = InternalApi()
+    api = _get_cling_api()
     if api.api_key is None:
         wandb.termlog("Login to W&B to sync offline runs")
         ctx.invoke(login, no_offline=True)
+        api = _get_cling_api(reset=True)
 
     if ignore:
         exclude_globs = ignore
@@ -451,27 +468,46 @@ def sync(
         exclude_globs = exclude_globs.split(",")
 
     def _summary():
-        sync_items = get_runs()
+        all_items = get_runs(
+            include_online=True,
+            include_offline=True,
+            include_synced=True,
+            include_unsynced=True,
+        )
+        sync_items = get_runs(
+            include_online=include_online if include_online is not None else True,
+            include_offline=include_offline if include_offline is not None else True,
+            include_synced=include_synced if include_synced is not None else False,
+            include_unsynced=True,
+            exclude_globs=exclude_globs,
+            include_globs=include_globs,
+        )
         synced = []
         unsynced = []
-        for item in sync_items:
+        for item in all_items:
             (synced if item.synced else unsynced).append(item)
-        if synced:
-            wandb.termlog("Number of synced runs: {}".format(len(synced)))
-        if unsynced:
-            wandb.termlog("Number of runs to be synced: {}".format(len(unsynced)))
-            if show and show < len(unsynced):
-                wandb.termlog("Showing {} unsynced runs:".format(show))
-            for item in unsynced[: (show or len(unsynced))]:
+        if sync_items:
+            wandb.termlog("Number of runs to be synced: {}".format(len(sync_items)))
+            if show and show < len(sync_items):
+                wandb.termlog("Showing {} runs to be synced:".format(show))
+            for item in sync_items[: (show or len(sync_items))]:
                 wandb.termlog("  {}".format(item))
+        else:
+            wandb.termlog("No runs to be synced.")
         if synced:
-            if not clean:
-                wandb.termlog(
-                    "NOTE: use sync --clean to cleanup synced runs from local directory."
+            clean_cmd = click.style("wandb sync --clean", fg="yellow")
+            wandb.termlog(
+                "NOTE: use {} to delete {} synced runs from local directory.".format(
+                    clean_cmd, len(synced)
                 )
+            )
         if unsynced:
-            if not path and not sync_all:
-                wandb.termlog("NOTE: use sync --sync-all to sync all unsynced runs.")
+            sync_cmd = click.style("wandb sync --sync-all", fg="yellow")
+            wandb.termlog(
+                "NOTE: use {} to sync {} unsynced runs from local directory.".format(
+                    sync_cmd, len(unsynced)
+                )
+            )
 
     def _sync_path(path):
         if run_id and len(path) > 1:
@@ -495,9 +531,10 @@ def sync(
 
     def _sync_all():
         sync_items = get_runs(
-            include_online=include_online,
-            include_offline=include_offline,
-            include_synced=include_synced,
+            include_online=include_online if include_online is not None else True,
+            include_offline=include_offline if include_offline is not None else True,
+            include_synced=include_synced if include_synced is not None else False,
+            include_unsynced=True,
             exclude_globs=exclude_globs,
             include_globs=include_globs,
         )
@@ -522,9 +559,9 @@ def sync(
             click.echo(click.style("Success!", fg="green"))
             return
         runs = get_runs(
-            include_online=True,
-            include_offline=True,
-            include_synced=True,
+            include_online=include_online if include_online is not None else True,
+            include_offline=include_offline if include_offline is not None else True,
+            include_synced=include_synced if include_synced is not None else True,
             include_unsynced=False,
             exclude_globs=exclude_globs,
             include_globs=include_globs,
@@ -608,10 +645,11 @@ def sweep(
         wandb.termwarn("Unable to parse settings parameter", repeat=False)
         return ret
 
-    api = InternalApi()
+    api = _get_cling_api()
     if api.api_key is None:
         wandb.termlog("Login to W&B to use the sweep feature")
         ctx.invoke(login, no_offline=True)
+        api = _get_cling_api(reset=True)
 
     sweep_obj_id = None
     if update:
@@ -673,11 +711,17 @@ def sweep(
             return
 
     env = os.environ
-    entity = entity or env.get("WANDB_ENTITY") or config.get("entity")
+    entity = (
+        entity
+        or env.get("WANDB_ENTITY")
+        or config.get("entity")
+        or api.settings("entity")
+    )
     project = (
         project
         or env.get("WANDB_PROJECT")
         or config.get("project")
+        or api.settings("project")
         or util.auto_project_name(config.get("program"))
     )
     sweep_id = api.upsert_sweep(
@@ -731,13 +775,14 @@ def sweep(
 @click.argument("sweep_id")
 @display_error
 def agent(ctx, project, entity, count, sweep_id):
-    api = InternalApi()
+    api = _get_cling_api()
     if api.api_key is None:
         wandb.termlog("Login to W&B to use the sweep agent feature")
         ctx.invoke(login, no_offline=True)
+        api = _get_cling_api(reset=True)
 
     wandb.termlog("Starting wandb agent 🕵️")
-    wandb_agent.run_agent(sweep_id, entity=entity, project=project, count=count)
+    wandb_agent.agent(sweep_id, entity=entity, project=project, count=count)
 
     # you can send local commands like so:
     # agent_api.command({'type': 'run', 'program': 'train.py',
@@ -1261,7 +1306,6 @@ def restore(ctx, run, no_git, branch, project, entity):
     commit, json_config, patch_content, metadata = api.run_config(
         project, run=run, entity=entity
     )
-    print(metadata)
     repo = metadata.get("git", {}).get("repo")
     image = metadata.get("docker")
     restore_message = (
@@ -1280,6 +1324,7 @@ Run `git clone %s` and restore from there or pass the --no-git flag."""
             )
 
     if commit and api.git.enabled:
+        wandb.termlog("Fetching origin and finding commit: {}".format(commit))
         subprocess.check_call(["git", "fetch", "--all"])
         try:
             api.git.repo.commit(commit)
@@ -1333,10 +1378,15 @@ Run `git clone %s` and restore from there or pass the --no-git flag."""
             patch_rel_path = os.path.relpath(patch_path, start=root)
             # --reject is necessary or else this fails any time a binary file
             # occurs in the diff
-            # we use .call() instead of .check_call() for the same reason
-            # TODO(adrian): this means there is no error checking here
-            subprocess.call(["git", "apply", "--reject", patch_rel_path], cwd=root)
-            wandb.termlog("Applied patch")
+            exit_code = subprocess.call(
+                ["git", "apply", "--reject", patch_rel_path], cwd=root
+            )
+            if exit_code == 0:
+                wandb.termlog("Applied patch")
+            else:
+                wandb.termerror(
+                    "Failed to apply patch, try un-staging any un-committed changes"
+                )
 
     util.mkdir_exists_ok(wandb_dir())
     config_path = os.path.join(wandb_dir(), "config.yaml")
@@ -1437,3 +1487,31 @@ def off():
         click.echo(
             "Unable to write config, copy and paste the following in your terminal to turn off W&B:\nexport WANDB_MODE=dryrun"
         )
+
+
+@cli.command("online", hidden=True)
+@click.pass_context
+@display_error
+def online(ctx):
+    ctx.invoke(on)
+
+
+@cli.command("offline", hidden=True)
+@click.pass_context
+@display_error
+def offline(ctx):
+    ctx.invoke(off)
+
+
+@cli.command("status", help="Show configuration settings")
+@click.option(
+    "--settings/--no-settings", help="Show the current settings", default=True
+)
+def status(settings):
+    api = _get_cling_api()
+    if settings:
+        click.echo(click.style("Current Settings", bold=True))
+        settings = api.settings()
+        click.echo(
+            json.dumps(settings, sort_keys=True, indent=2, separators=(",", ": "))
+        )
