@@ -13,6 +13,7 @@ these values in many cases.
 import os
 import sys
 import json
+import wandb
 from distutils.util import strtobool
 
 CONFIG_PATHS = 'WANDB_CONFIG_PATHS'
@@ -58,8 +59,10 @@ HOST = 'WANDB_HOST'
 ANONYMOUS = 'WANDB_ANONYMOUS'
 JUPYTER = 'WANDB_JUPYTER'
 CONFIG_DIR = 'WANDB_CONFIG_DIR'
-PROGRAM = 'WANDB_PROGRAM'
-WANDB_CACHE_DIR = 'WANDB_CACHE_DIR'
+CACHE_DIR = 'WANDB_CACHE_DIR'
+
+# For testing, to be removed in future version
+USE_V1_ARTIFACTS = '_WANDB_USE_V1_ARTIFACTS'
 
 
 def immutable_keys():
@@ -68,7 +71,7 @@ def immutable_keys():
     return [DIR, ENTITY, PROJECT, API_KEY, IGNORE, DISABLE_CODE, DOCKER, MODE, BASE_URL,
             ERROR_REPORTING, CRASH_NOSYNC_TIME, MAGIC, USERNAME, USER_EMAIL, DIR, SILENT, CONFIG_PATHS,
             ANONYMOUS, RUN_GROUP, JOB_TYPE, TAGS, RESUME, AGENT_REPORT_INTERVAL, HTTP_TIMEOUT,
-            HOST, SAVE_CODE, WANDB_CACHE_DIR]
+            HOST, CACHE_DIR, USE_V1_ARTIFACTS]
 
 
 def _env_as_bool(var, default=None, env=None):
@@ -89,11 +92,6 @@ def is_debug(default=None, env=None):
 def error_reporting_enabled():
     return _env_as_bool(ERROR_REPORTING, default=True)
 
-def should_save_code():
-    save_code = _env_as_bool(SAVE_CODE, default=False)
-    code_disabled = _env_as_bool(DISABLE_CODE, default=False)
-    # SAVE_CODE takes precedence over DISABLE_CODE
-    return save_code and not code_disabled
 
 def get_error_reporting(default=True, env=None):
     if env is None:
@@ -207,8 +205,10 @@ def get_dir(default=None, env=None):
     return env.get(DIR, default)
 
 
-def get_config_paths():
-    pass
+def get_config_paths(default=None, env=None):
+    if env is None:
+        env = os.environ
+    return env.get(CONFIG_PATHS, default)
 
 
 def get_agent_report_interval(default=None, env=None):
@@ -250,13 +250,22 @@ def get_magic(default=None, env=None):
     val = env.get(MAGIC, default)
     return val
 
+
 def get_cache_dir(env=None):
-    DEFAULT_DIR = os.path.expanduser(os.path.join('~', '.cache', 'wandb'))
+    default_dir = os.path.expanduser(os.path.join('~', '.cache', 'wandb'))
     if env is None:
         env = os.environ
-    val = env.get(WANDB_CACHE_DIR, DEFAULT_DIR)
+    val = env.get(CACHE_DIR, default_dir)
     return val
 
+
+def get_use_v1_artifacts(env=None):
+    if env is None:
+        env = os.environ
+    val = env.get(USE_V1_ARTIFACTS, False)
+    return val
+
+
 def set_entity(value, env=None):
     if env is None:
         env = os.environ
@@ -267,3 +276,10 @@ def set_project(value, env=None):
     if env is None:
         env = os.environ
     env[PROJECT] = value or "uncategorized"
+
+
+def should_save_code():
+    save_code = _env_as_bool(SAVE_CODE, default=False)
+    code_disabled = _env_as_bool(DISABLE_CODE, default=False)
+    # SAVE_CODE takes precedence over DISABLE_CODE
+    return save_code and not code_disabled
