@@ -5,16 +5,17 @@ import threading
 from six.moves import queue
 
 from wandb.filesync import upload_job
-from wandb import termerror
+from wandb.errors.term import termerror
 
 
 RequestUpload = collections.namedtuple(
-    'EventStartUploadJob', ('path', 'save_name', 'artifact_id', 'md5', 'copied', 'save_fn', 'digest'))
+    'EventStartUploadJob', ('path', 'save_name', 'artifact_id', 'md5', 'copied',
+                            'save_fn', 'digest'))
 RequestCommitArtifact = collections.namedtuple(
-    'RequestCommitArtifact', ('artifact_id', 'on_commit'))
+    'RequestCommitArtifact', ('artifact_id', 'before_commit', 'on_commit'))
 RequestFinish = collections.namedtuple('RequestFinish', ())
 
-    
+
 class StepUpload(object):
     def __init__(self, api, stats, event_queue, max_jobs):
         self._api = api
@@ -76,18 +77,18 @@ class StepUpload(object):
                 event = self._pending_jobs.pop(0)
                 self._start_upload_job(event)
         elif isinstance(event, RequestCommitArtifact):
+            if event.artifact_id not in self._artifacts:
+                self._init_artifact(event.artifact_id)
             self._artifacts[event.artifact_id]['commit_requested'] = True
+            if event.before_commit:
+                self._artifacts[event.artifact_id]['pre_commit_callbacks'].add(event.before_commit)
             if event.on_commit:
-                self._artifacts[event.artifact_id]['commit_callbacks'].add(event.on_commit)
+                self._artifacts[event.artifact_id]['post_commit_callbacks'].add(event.on_commit)
             self._maybe_commit_artifact(event.artifact_id)
         elif isinstance(event, RequestUpload):
             if event.artifact_id is not None:
                 if event.artifact_id not in self._artifacts:
-                    self._artifacts[event.artifact_id] = {
-                        'pending_count': 0,
-                        'commit_requested': False,
-                        'commit_callbacks': set(),
-                    }
+                    self._init_artifact(event.artifact_id)
                 self._artifacts[event.artifact_id]['pending_count'] += 1
             if len(self._running_jobs) == self._max_jobs:
                 self._pending_jobs.append(event)
@@ -115,11 +116,21 @@ class StepUpload(object):
         self._running_jobs[event.save_name] = job
         job.start()
 
+    def _init_artifact(self, artifact_id):
+        self._artifacts[artifact_id] = {
+            'pending_count': 0,
+            'commit_requested': False,
+            'pre_commit_callbacks': set(),
+            'post_commit_callbacks': set(),
+        }
+
     def _maybe_commit_artifact(self, artifact_id):
         artifact_status = self._artifacts[artifact_id]
         if artifact_status['pending_count'] == 0 and artifact_status['commit_requested']:
+            for callback in artifact_status['pre_commit_callbacks']:
+                callback()
             self._api.commit_artifact(artifact_id)
-            for callback in artifact_status['commit_callbacks']:
+            for callback in artifact_status['post_commit_callbacks']:
                 callback()
 
     def start(self):
