@@ -321,6 +321,12 @@ class WandbCallback(keras.callbacks.Callback):
             else:
                 self.monitor_op = operator.lt
                 self.best = float("inf")
+        # Get the previous best metric for resumed runs
+        previous_best = wandb.run.summary.get(
+            "%s%s" % (self.log_best_prefix, self.monitor)
+        )
+        if previous_best is not None:
+            self.best = previous_best
 
     def _implements_train_batch_hooks(self):
         return self.log_batch_frequency is not None
