@@ -20,12 +20,18 @@ class Unbuffered(object):
         self.stream = stream
 
     def write(self, data):
-        self.stream.write(data)
-        self.stream.flush()
+        try:
+            self.stream.write(data)
+            self.stream.flush()
+        except Exception:
+            pass  # Underlying stream might be closed.
 
     def writelines(self, datas):
-        self.stream.writelines(datas)
-        self.stream.flush()
+        try:
+            self.stream.writelines(datas)
+            self.stream.flush()
+        except Exception:
+            pass  # Underlying stream might be closed.
 
     def __getattr__(self, attr):
         return getattr(self.stream, attr)
@@ -158,7 +164,10 @@ class Redirect(object):
             fp = getattr(sys, self._stream)
             # TODO(jhr): does this still work under windows?  are we leaking a fd?
             # Do not close old filedescriptor as others might be using it
-            fp.close()
+            try:
+                fp.close()
+            except Exception:
+                pass  # Stream might be wrapped by another program which doesn't support closing.
         os.dup2(to_fd, self._old_fd)
         if self._io_wrapped:
             if close:
