@@ -22,7 +22,6 @@ from wandb.integration.magic import magic_install
 from wandb.lib import filesystem, module, reporting
 from wandb.util import sentry_exc
 
-from . import wandb_login
 from . import wandb_setup
 from .wandb_helper import parse_config
 from .wandb_run import Run, RunBase, RunDummy
@@ -70,8 +69,7 @@ class _WandbInit(object):
         _set_logger(self._wl._get_logger())
 
         # Start with settings from wandb library singleton
-        settings: Settings = self._wl.settings().duplicate()
-
+        settings: Settings = self._wl._clone_settings()
         settings_param = kwargs.pop("settings", None)
         if settings_param:
             settings._apply_settings(settings_param)
@@ -88,6 +86,7 @@ class _WandbInit(object):
                 if sm_api_key:
                     sm_env["WANDB_API_KEY"] = sm_api_key
                 settings._apply_environ(sm_env)
+                wandb.setup(settings=settings)
             for k, v in six.iteritems(sm_run):
                 kwargs.setdefault(k, v)
 
@@ -113,18 +112,6 @@ class _WandbInit(object):
         for k, v in init_config.items():
             self.config.setdefault(k, v)
 
-        # Temporarily unsupported parameters
-        unsupported = (
-            "allow_val_change",
-            "force",
-        )
-        for key in unsupported:
-            val = kwargs.pop(key, None)
-            if val:
-                self._reporter.warning(
-                    "currently unsupported wandb.init() arg: %s", key
-                )
-
         monitor_gym = kwargs.pop("monitor_gym", None)
         if monitor_gym and len(wandb.patched["gym"]) == 0:
             wandb.gym.monitor()
@@ -138,20 +125,18 @@ class _WandbInit(object):
         if magic not in (None, False):
             magic_install(kwargs)
 
-        # prevent setting project, entity if in sweep
-        # TODO(jhr): these should be locked elements in the future or at least
-        #            moved to apply_init()
-        if settings.sweep_id:
-            for key in ("project", "entity"):
-                val = kwargs.pop(key, None)
-                if val:
-                    print("Ignored wandb.init() arg %s when running a sweep" % key)
-        settings.apply_init(kwargs)
-
-        login_key = wandb_login._login(_disable_warning=True, _settings=settings)
+        # handle login related parameters as these are applied to global state
+        anonymous = kwargs.pop("anonymous", None)
+        force = kwargs.pop("force", None)
+        login_key = wandb.login(anonymous=anonymous, force=force)
         if not login_key:
             settings.mode = "offline"
 
+        # apply updated global state after login was handled
+        settings._apply_settings(wandb.setup()._settings)
+
+        settings._apply_init(kwargs)
+
         # TODO(jhr): should this be moved? probably.
         d = dict(_start_time=time.time(), _start_datetime=datetime.datetime.now(),)
         settings.update(d)
@@ -320,13 +305,12 @@ class _WandbInit(object):
                 run=run,
                 config=run.config,
                 log=run.log,
-                join=run.join,
-                finish=run.finish,
                 summary=run.summary,
                 save=run.save,
                 restore=run.restore,
                 use_artifact=run.use_artifact,
                 log_artifact=run.log_artifact,
+                plot_table=run.plot_table,
             )
             return run
 
@@ -354,9 +338,7 @@ class _WandbInit(object):
         )
         backend.server_connect()
         # Make sure we are logged in
-        wandb_login._login(
-            _backend=backend, _disable_warning=True, _settings=self.settings
-        )
+        # wandb_login._login(_backend=backend, _settings=self.settings)
 
         # resuming needs access to the server, check server_status()?
 
@@ -408,13 +390,12 @@ class _WandbInit(object):
             run=run,
             config=run.config,
             log=run.log,
-            join=run.join,
-            finish=run.finish,
             summary=run.summary,
             save=run.save,
             restore=run.restore,
             use_artifact=run.use_artifact,
             log_artifact=run.log_artifact,
+            plot_table=run.plot_table,
         )
         self._reporter.set_context(run=run)
         run._on_start()
@@ -446,14 +427,15 @@ def init(
     config_include_keys=None,
     anonymous: Optional[str] = None,
     mode: Optional[str] = None,
-    allow_val_change: bool = None,
+    allow_val_change: Optional[bool] = None,
     resume: Optional[Union[bool, str]] = None,
-    force=None,
+    force: Optional[bool] = None,
     tensorboard=None,  # alias for sync_tensorboard
     sync_tensorboard=None,
     monitor_gym=None,
+    save_code=None,
     id=None,
-    settings: Union[Settings, Dict[str, Any], str, None] = None,
+    settings: Union[Settings, Dict[str, Any], None] = None,
 ) -> RunBase:
     """Initialize a wandb Run.
 
@@ -483,7 +465,10 @@ def init(
         except (KeyboardInterrupt, Exception) as e:
             if not isinstance(e, KeyboardInterrupt):
                 sentry_exc(e)
-            getcaller()
+            if not (
+                wandb.wandb_agent._is_running() and isinstance(e, KeyboardInterrupt)
+            ):
+                getcaller()
             assert logger
             if wi.settings.problem == "fatal":
                 raise
@@ -496,7 +481,7 @@ def init(
     except KeyboardInterrupt as e:
         assert logger
         logger.warning("interrupted", exc_info=e)
-        six.raise_from(Exception("interrupted"), e)
+        raise e
     except Exception as e:
         error_seen = e
         traceback.print_exc()
