@@ -31,9 +31,22 @@ from wandb.apis import internal, public
 from wandb.data_types import _datatypes_set_callback
 from wandb.errors import Error
 from wandb.interface.summary_record import SummaryRecord
-from wandb.lib import filenames, module, proto_util, redirect, sparkline
+from wandb.lib import (
+    config_util,
+    filenames,
+    ipython,
+    module,
+    proto_util,
+    redirect,
+    sparkline,
+)
 from wandb.util import add_import_hook, sentry_set_scope, to_forward_slash_path
-from wandb.viz import Visualize
+from wandb.viz import (
+    create_custom_chart,
+    custom_chart_panel_config,
+    CustomChart,
+    Visualize,
+)
 
 from . import wandb_config
 from . import wandb_history
@@ -44,6 +57,7 @@ if wandb.TYPE_CHECKING:  # type: ignore
 
 logger = logging.getLogger("wandb")
 EXIT_TIMEOUT = 60
+RUN_NAME_COLOR = "#cdcd00"
 
 
 class ExitHooks(object):
@@ -156,12 +170,20 @@ class RunDummy(RunBase):
     def id(self):
         pass
 
+    @property
+    def sweep_id(self):
+        pass
+
     def get_url(self):
         pass
 
     def project_name(self):
         pass
 
+    @property
+    def project(self):
+        pass
+
     @property
     def config(self):
         return self._config
@@ -210,6 +232,12 @@ class RunDummy(RunBase):
     def use_artifact(self, artifact_or_name, type=None, aliases=None):
         pass
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_val, exc_tb):
+        return exc_type is None
+
 
 class Run(RunBase):
     def __init__(self, config=None, settings=None):
@@ -257,6 +285,9 @@ class Run(RunBase):
         self._exit_result = None
         self._final_summary = None
         self._sampled_history = None
+        self._jupyter_progress = None
+        if self._settings._jupyter:
+            self._jupyter_progress = ipython.jupyter_progress_bar()
 
         self._output_writer = None
 
@@ -265,7 +296,12 @@ class Run(RunBase):
 
         # Initial scope setup for sentry. This might get changed when the
         # actual run comes back.
-        sentry_set_scope("user", self._entity, self._project)
+        sentry_set_scope(
+            "user",
+            entity=self._entity,
+            project=self._project,
+            email=self._settings.email,
+        )
 
         # Returned from backend request_run(), set from wandb_init?
         self._run_obj = None
@@ -389,6 +425,10 @@ class Run(RunBase):
     def config(self):
         return self._config
 
+    @property
+    def config_static(self):
+        return wandb_config.ConfigStatic(self._config)
+
     @property
     def name(self):
         if self._name:
@@ -421,9 +461,8 @@ class Run(RunBase):
     def tags(self) -> Optional[Tuple]:
         if self._tags:
             return self._tags
-        if not self._run_obj:
-            return None
-        return self._run_obj.tags
+        run_obj = self._run_obj or self._run_obj_offline
+        return run_obj.tags
 
     @tags.setter
     def tags(self, tags: Sequence):
@@ -435,6 +474,12 @@ class Run(RunBase):
     def id(self):
         return self._run_id
 
+    @property
+    def sweep_id(self):
+        if not self._run_obj:
+            return None
+        return self._run_obj.sweep_id or None
+
     @property
     def path(self):
         parts = []
@@ -459,7 +504,7 @@ class Run(RunBase):
 
     @property
     def resumed(self):
-        return self._starting_step > 0
+        return self.starting_step > 0
 
     @property
     def step(self):
@@ -469,12 +514,47 @@ class Run(RunBase):
         run_obj = self._run_obj or self._run_obj_offline
         return run_obj.project
 
+    @property
+    def mode(self):
+        """For compatibility with 0.9.x and earlier, deprecate eventually."""
+        return "dryrun" if self._settings._offline else "run"
+
+    @property
+    def offline(self):
+        return self._settings._offline
+
+    @property
+    def group(self):
+        run_obj = self._run_obj or self._run_obj_offline
+        return run_obj.run_group
+
+    @property
+    def job_type(self):
+        run_obj = self._run_obj or self._run_obj_offline
+        return run_obj.job_type
+
+    @property
+    def project(self):
+        return self.project_name()
+
     def get_url(self):
         if not self._run_obj:
             wandb.termwarn("URL not available in offline run")
             return
         return self._get_run_url()
 
+    def get_project_url(self):
+        if not self._run_obj:
+            wandb.termwarn("URL not available in offline run")
+            return
+        return self._get_project_url()
+
+    def get_sweep_url(self):
+        if not self._run_obj:
+            wandb.termwarn("URL not available in offline run")
+            return
+        return self._get_sweep_url()
+
     @property
     def url(self):
         return self.get_url()
@@ -521,6 +601,7 @@ class Run(RunBase):
 
         # TODO(jhr): move visualize hack somewhere else
         visualize_persist_config = False
+        custom_charts = {}
         for k in row:
             if isinstance(row[k], Visualize):
                 if "viz" not in self._config["_wandb"]:
@@ -531,6 +612,23 @@ class Run(RunBase):
                 }
                 row[k] = row[k].value
                 visualize_persist_config = True
+            elif isinstance(row[k], CustomChart):
+                custom_charts[k] = row[k]
+                custom_chart = row[k]
+
+        for k, custom_chart in custom_charts.items():
+            # remove the chart key from the row
+            # TODO: is this really the right move? what if the user logs
+            #     a non-custom chart to this key?
+            row.pop(k)
+            # add the table under a different key
+            table_key = k + "_table"
+            row[table_key] = custom_chart.table
+            # add the panel
+            panel_config = custom_chart_panel_config(custom_chart, k, table_key)
+            self._add_panel(k, "Vega2", panel_config)
+            visualize_persist_config = True
+
         if visualize_persist_config:
             self._config_callback(data=self._config._as_dict())
 
@@ -559,10 +657,31 @@ class Run(RunBase):
 
     def _set_run_obj(self, run_obj):
         self._run_obj = run_obj
-        # TODO: Update run summary when resuming?
+        self._entity = run_obj.entity
+        self._project = run_obj.project
+        # Grab the config from resuming
+        if run_obj.config:
+            c_dict = config_util.dict_no_value_from_proto_list(run_obj.config.update)
+            # TODO: Windows throws a wild error when this is set...
+            if "_wandb" in c_dict:
+                del c_dict["_wandb"]
+            # We update the config object here without triggering the callback
+            self.config._update(c_dict, allow_val_change=True)
+        # Update the summary, this will trigger an un-needed graphql request :(
+        if run_obj.summary:
+            summary_dict = {}
+            for orig in run_obj.summary.update:
+                summary_dict[orig.key] = json.loads(orig.value_json)
+            self.summary.update(summary_dict)
         self.history._update_step()
         # TODO: It feels weird to call this twice..
-        sentry_set_scope("user", run_obj.entity, run_obj.project, self._get_run_url())
+        sentry_set_scope(
+            "user",
+            entity=run_obj.entity,
+            project=run_obj.project,
+            email=self._settings.email,
+            url=self._get_run_url(),
+        )
 
     def _set_run_obj_offline(self, run_obj):
         self._run_obj_offline = run_obj
@@ -852,7 +971,7 @@ class Run(RunBase):
             return None
         # if the file does not exist, the file has an md5 of 0
         if files[0].md5 == "0":
-            raise ValueError("File {} not found.".format(path))
+            raise ValueError("File {} not found in {}.".format(name, run_path or root))
         return files[0].download(root=root, replace=True)
 
     def finish(self, exit_code=None):
@@ -872,17 +991,44 @@ class Run(RunBase):
     def join(self, exit_code=None):
         self.finish(exit_code=exit_code)
 
+    def plot_table(self, vega_spec_name, data_table, fields, string_fields=None):
+        """Creates a custom plot on a table.
+        Args:
+            vega_spec_name: the name of the spec for the plot
+            table_key: the key used to log the data table
+            data_table: a wandb.Table object containing the data to
+                        be used on the visualization
+            fields: a dict mapping from table keys to fields that the custom
+                    visualization needs
+            string_fields: a dict that provides values for any string constants
+                    the custom visualization needs
+        """
+        visualization = create_custom_chart(
+            vega_spec_name, data_table, fields, string_fields or {}
+        )
+        return visualization
+
+    def _add_panel(self, visualize_key: str, panel_type: str, panel_config: dict):
+        if "visualize" not in self._config["_wandb"]:
+            self._config["_wandb"]["visualize"] = dict()
+        self._config["_wandb"]["visualize"][visualize_key] = {
+            "panel_type": panel_type,
+            "panel_config": panel_config,
+        }
+
+        self._config_callback(data=self._config._as_dict())
+
     def _get_project_url(self):
         s = self._settings
         r = self._run_obj
-        app_url = s.base_url.replace("//api.", "//app.")
+        app_url = wandb.util.app_url(s.base_url)
         url = "{}/{}/{}".format(app_url, url_quote(r.entity), url_quote(r.project))
         return url
 
     def _get_run_url(self):
         s = self._settings
         r = self._run_obj
-        app_url = s.base_url.replace("//api.", "//app.")
+        app_url = wandb.util.app_url(s.base_url)
         url = "{}/{}/{}/runs/{}".format(
             app_url, url_quote(r.entity), url_quote(r.project), url_quote(r.run_id)
         )
@@ -901,7 +1047,7 @@ class Run(RunBase):
         if not sweep_id:
             return
 
-        app_url = self._settings.base_url.replace("//api.", "//app.")
+        app_url = wandb.util.app_url(self._settings.base_url)
 
         return "{base}/{entity}/{project}/sweeps/{sweepid}".format(
             base=app_url,
@@ -918,9 +1064,19 @@ class Run(RunBase):
         project_url = self._get_project_url()
         run_url = self._get_run_url()
         sweep_url = self._get_sweep_url()
-        if self._settings._jupyter:
-            from IPython.core.display import display, HTML  # type: ignore
+        version_str = "Tracking run with wandb version {}".format(wandb.__version__)
+        if self.resumed:
+            run_state_str = "Resuming run"
+        else:
+            run_state_str = "Syncing run"
+        run_name = self._get_run_name()
+        app_url = wandb.util.app_url(self._settings.base_url)
 
+        sync_dir = self._settings._sync_dir
+        if self._settings._jupyter:
+            sync_dir = "<code>{}</code>".format(sync_dir)
+        dir_str = "Run data is saved locally in {}".format(sync_dir)
+        if self._settings._jupyter:
             sweep_line = (
                 'Sweep page: <a href="{}" target="_blank">{}</a><br/>\n'.format(
                     sweep_url, sweep_url
@@ -929,23 +1085,33 @@ class Run(RunBase):
                 else ""
             )
             docs_html = '<a href="https://docs.wandb.com/integrations/jupyter.html" target="_blank">(Documentation)</a>'  # noqa: E501
-            display(
-                HTML(
-                    """
-                Logging results to <a href="https://wandb.com" target="_blank">Weights & Biases</a> {}.<br/>
+            ipython.display_html(
+                """
+                {}<br/>
+                {} <strong style="color:{}">{}</strong> to <a href="{}" target="_blank">Weights & Biases</a> {}.<br/>
                 Project page: <a href="{}" target="_blank">{}</a><br/>
                 {}Run page: <a href="{}" target="_blank">{}</a><br/>
+                {}<br/><br/>
             """.format(  # noqa: E501
-                        docs_html,
-                        project_url,
-                        project_url,
-                        sweep_line,
-                        run_url,
-                        run_url,
-                    )
+                    version_str,
+                    run_state_str,
+                    RUN_NAME_COLOR,
+                    run_name,
+                    app_url,
+                    docs_html,
+                    project_url,
+                    project_url,
+                    sweep_line,
+                    run_url,
+                    run_url,
+                    dir_str,
                 )
             )
         else:
+            wandb.termlog(version_str)
+            wandb.termlog(
+                "{} {}".format(run_state_str, click.style(run_name, fg="yellow"))
+            )
             emojis = dict(star="", broom="", rocket="")
             if platform.system() != "Windows":
                 emojis = dict(star="⭐️", broom="🧹", rocket="🚀")
@@ -969,8 +1135,10 @@ class Run(RunBase):
                     click.style(run_url, underline=True, fg="blue"),
                 )
             )
+            wandb.termlog(dir_str)
             if not self._settings._offline:
                 wandb.termlog("Run `wandb off` to turn off syncing.")
+            print("")
 
     def _redirect(self, stdout_slave_fd, stderr_slave_fd, console=None):
         if console is None:
@@ -1087,17 +1255,22 @@ class Run(RunBase):
         self._exit_code = exit_code
         try:
             self._on_finish()
-        except KeyboardInterrupt:
+        except KeyboardInterrupt as ki:
+            if wandb.wandb_agent._is_running():
+                raise ki
             wandb.termerror("Control-C detected -- Run data was not synced")
-            os._exit(-1)
+            if ipython._get_python_type() == "python":
+                os._exit(-1)
         except Exception as e:
             self._console_stop()
             self._backend.cleanup()
             logger.error("Problem finishing run", exc_info=e)
             wandb.termerror("Problem finishing run")
             traceback.print_exception(*sys.exc_info())
-            os._exit(-1)
-        self._on_final()
+            if ipython._get_python_type() == "python":
+                os._exit(-1)
+        else:
+            self._on_final()
 
     def _console_start(self):
         logger.info("atexit reg")
@@ -1115,13 +1288,15 @@ class Run(RunBase):
 
     def _console_stop(self):
         self._restore()
-        self._output_writer.close()
-        self._output_writer = None
+        if self._output_writer:
+            self._output_writer.close()
+            self._output_writer = None
 
     def _on_start(self):
+        # TODO: make offline mode in jupyter use HTML
         if self._settings._offline:
             wandb.termlog("Offline run mode, not syncing to the cloud.")
-        wandb.termlog("Tracking run with wandb version {}".format(wandb.__version__))
+
         if self._settings._offline:
             wandb.termlog(
                 (
@@ -1129,20 +1304,8 @@ class Run(RunBase):
                     "Run `wandb on` to enable cloud syncing."
                 )
             )
-        wandb.termlog(
-            "Run data is saved locally in {}".format(self._settings._sync_dir)
-        )
         if self._run_obj:
-            if self.resumed:
-                run_state_str = "Resuming run"
-            else:
-                run_state_str = "Syncing run"
-            run_name = self._get_run_name()
-            wandb.termlog(
-                "{} {}".format(run_state_str, click.style(run_name, fg="yellow"))
-            )
             self._display_run()
-        print("")
         if self._backend and not self._settings._offline:
             self._run_status_checker = RunStatusChecker(self._backend.interface)
         self._console_start()
@@ -1150,30 +1313,42 @@ class Run(RunBase):
     def _pusher_print_status(self, progress, prefix=True, done=False):
         if self._settings._offline:
             return
-        spinner_states = ["-", "\\", "|", "/"]
+
         line = " %.2fMB of %.2fMB uploaded (%.2fMB deduped)\r" % (
             progress.uploaded_bytes / 1048576.0,
             progress.total_bytes / 1048576.0,
             progress.deduped_bytes / 1048576.0,
         )
-        line = spinner_states[self._progress_step % 4] + line
-        self._progress_step += 1
-        wandb.termlog(line, newline=False, prefix=prefix)
 
-        if done:
-            dedupe_fraction = (
-                progress.deduped_bytes / float(progress.total_bytes)
-                if progress.total_bytes > 0
-                else 0
-            )
-            if dedupe_fraction > 0.01:
-                wandb.termlog(
-                    "W&B sync reduced upload amount by %.1f%%             "
-                    % (dedupe_fraction * 100),
-                    prefix=prefix,
+        if self._jupyter_progress:
+            if progress.total_bytes == 0:
+                percent_done = 1
+            else:
+                percent_done = progress.uploaded_bytes / progress.total_bytes
+            self._jupyter_progress.update(percent_done, line)
+            if done:
+                self._jupyter_progress.close()
+        elif not self._settings._jupyter:
+            spinner_states = ["-", "\\", "|", "/"]
+
+            line = spinner_states[self._progress_step % 4] + line
+            self._progress_step += 1
+            wandb.termlog(line, newline=False, prefix=prefix)
+
+            if done:
+                dedupe_fraction = (
+                    progress.deduped_bytes / float(progress.total_bytes)
+                    if progress.total_bytes > 0
+                    else 0
                 )
-            # clear progress line.
-            wandb.termlog(" " * 79, prefix=prefix)
+                if dedupe_fraction > 0.01:
+                    wandb.termlog(
+                        "W&B sync reduced upload amount by %.1f%%             "
+                        % (dedupe_fraction * 100),
+                        prefix=prefix,
+                    )
+                # clear progress line.
+                wandb.termlog(" " * 79, prefix=prefix)
 
     def _on_finish_progress(self, progress, done=None):
         self._pusher_print_status(progress, done=done)
@@ -1202,17 +1377,21 @@ class Run(RunBase):
         # make sure all uncommitted history is flushed
         self.history._flush()
 
-        self._console_stop()
-        print("")
+        self._console_stop()  # TODO: there's a race here with jupyter console logging
         pid = self._backend._internal_pid
-        wandb.termlog("Waiting for W&B process to finish, PID {}".format(pid))
+
+        status_str = "Waiting for W&B process to finish, PID {}".format(pid)
         if not self._exit_code:
-            wandb.termlog("Program ended successfully.")
+            status_str += "\nProgram ended successfully."
         else:
-            msg = "Program failed with code {}. ".format(self._exit_code)
+            status_str += "\nProgram failed with code {}. ".format(self._exit_code)
             if not self._settings._offline:
-                msg += " Press ctrl-c to abort syncing."
-            wandb.termlog(msg)
+                status_str += " Press ctrl-c to abort syncing."
+        if self._settings._jupyter:
+            ipython.display_html("<br/>" + status_str.replace("\n", "<br/>"))
+        else:
+            print("")
+            wandb.termlog(status_str)
 
         # TODO: we need to handle catastrophic failure better
         # some tests were timing out on sending exit for reasons not clear to me
@@ -1237,10 +1416,8 @@ class Run(RunBase):
 
     def _on_final(self):
         # check for warnings and errors, show log file locations
-        # if self._run_obj:
-        #    self._display_run()
-        # print("DEBUG on finish")
         if self._reporter:
+            # TODO: handle warnings and errors nicely in jupyter
             warning_lines = self._reporter.warning_lines
             if warning_lines:
                 wandb.termlog("Warnings:")
@@ -1257,15 +1434,23 @@ class Run(RunBase):
                 if len(error_lines) < self._reporter.error_count:
                     wandb.termlog("More errors")
         if self._settings.log_user:
-            wandb.termlog(
-                "Find user logs for this run at: {}".format(self._settings.log_user)
-            )
+            log_user = self._settings.log_user
+            if self._settings._jupyter:
+                log_user = "<code>{}</code>".format(log_user)
+            log_str = "Find user logs for this run at: {}".format(log_user)
+            if self._settings._jupyter:
+                ipython.display_html(log_str)
+            else:
+                wandb.termlog(log_str)
         if self._settings.log_internal:
-            wandb.termlog(
-                "Find internal logs for this run at: {}".format(
-                    self._settings.log_internal
-                )
-            )
+            log_internal = self._settings.log_internal
+            if self._settings._jupyter:
+                log_internal = "<code>{}</code>".format(log_internal)
+            log_str = "Find internal logs for this run at: {}".format(log_internal)
+            if self._settings._jupyter:
+                ipython.display_html(log_str)
+            else:
+                wandb.termlog(log_str)
         self._show_summary()
         self._show_history()
         self._show_files()
@@ -1273,13 +1458,24 @@ class Run(RunBase):
         if self._run_obj:
             run_url = self._get_run_url()
             run_name = self._get_run_name()
-            wandb.termlog(
-                "\nSynced {}: {}".format(
-                    click.style(run_name, fg="yellow"), click.style(run_url, fg="blue")
+            if self._settings._jupyter:
+                ipython.display_html(
+                    """
+                    <br/>Synced <strong style="color:{}">{}</strong>: <a href="{}" target="_blank">{}</a><br/>
+                """.format(
+                        RUN_NAME_COLOR, run_name, run_url, run_url
+                    )
+                )
+            else:
+                wandb.termlog(
+                    "\nSynced {}: {}".format(
+                        click.style(run_name, fg="yellow"),
+                        click.style(run_url, fg="blue"),
+                    )
                 )
-            )
 
         if self._settings._offline:
+            # TODO: handle jupyter offline messages
             wandb.termlog("You can sync this run to the cloud by running:")
             wandb.termlog(
                 click.style(
@@ -1290,17 +1486,31 @@ class Run(RunBase):
     def _show_summary(self):
         if self._final_summary:
             logger.info("rendering summary")
-            wandb.termlog("Run summary:")
             max_len = max([len(k) for k in self._final_summary.keys()])
             format_str = "  {:>%s} {}" % max_len
+            summary_rows = []
             for k, v in iteritems(self._final_summary):
                 # arrays etc. might be too large. for now we just don't print them
                 if isinstance(v, string_types):
                     if len(v) >= 20:
                         v = v[:20] + "..."
-                    wandb.termlog(format_str.format(k, v))
+                    summary_rows.append((k, v))
                 elif isinstance(v, numbers.Number):
-                    wandb.termlog(format_str.format(k, v))
+                    if isinstance(v, float):
+                        v = round(v, 5)
+                    summary_rows.append((k, v))
+            if self._settings._jupyter:
+                summary_table = ipython.STYLED_TABLE_HTML
+                for row in summary_rows:
+                    summary_table += "<tr><td>{}</td><td>{}</td></tr>".format(*row)
+                summary_table += "</table>"
+                ipython.display_html("<h3>Run summary:</h3><br/>" + summary_table)
+            else:
+                summary_lines = "\n".join(
+                    [format_str.format(k, v) for k, v in summary_rows]
+                )
+                wandb.termlog("Run summary:")
+                wandb.termlog(summary_lines)
 
     def _show_history(self):
         if not self._sampled_history:
@@ -1314,15 +1524,27 @@ class Run(RunBase):
             return
 
         logger.info("rendering history")
-        wandb.termlog("Run history:")
         max_len = max([len(k) for k in self._sampled_history])
+        history_rows = []
         for key in self._sampled_history:
             vals = wandb.util.downsample(self._sampled_history[key], 40)
             if any((not isinstance(v, numbers.Number) for v in vals)):
                 continue
             line = sparkline.sparkify(vals)
-            format_str = u"  {:>%s} {}" % max_len
-            wandb.termlog(format_str.format(key, line))
+            history_rows.append((key, line))
+        if self._settings._jupyter:
+            history_table = ipython.STYLED_TABLE_HTML
+            for row in history_rows:
+                history_table += "<tr><td>{}</td><td>{}</td></tr>".format(*row)
+            history_table += "</table>"
+            ipython.display_html("<h3>Run history:</h3><br/>" + history_table + "<br/>")
+        else:
+            wandb.termlog("Run history:")
+            history_lines = ""
+            format_str = u"  {:>%s} {}\n" % max_len
+            for row in history_rows:
+                history_lines += format_str.format(*row)
+            wandb.termlog(history_lines)
 
     def _show_files(self):
         if not self._poll_exit_response or not self._poll_exit_response.file_counts:
@@ -1330,14 +1552,16 @@ class Run(RunBase):
         if self._settings._offline:
             return
         logger.info("logging synced files")
-        wandb.termlog(
-            "Synced {} W&B file(s), {} media file(s), {} artifact file(s) and {} other file(s)".format(  # noqa:E501
-                self._poll_exit_response.file_counts.wandb_count,
-                self._poll_exit_response.file_counts.media_count,
-                self._poll_exit_response.file_counts.artifact_count,
-                self._poll_exit_response.file_counts.other_count,
-            )
+        file_str = "Synced {} W&B file(s), {} media file(s), {} artifact file(s) and {} other file(s)".format(  # noqa:E501
+            self._poll_exit_response.file_counts.wandb_count,
+            self._poll_exit_response.file_counts.media_count,
+            self._poll_exit_response.file_counts.artifact_count,
+            self._poll_exit_response.file_counts.other_count,
         )
+        if self._settings._jupyter:
+            ipython.display_html(file_str)
+        else:
+            wandb.termlog(file_str)
 
     def _save_job_spec(self):
         envdict = dict(python="python3.6", requirements=[],)
@@ -1524,3 +1748,8 @@ class WriteSerializingFile(object):
             self.f.close()
         finally:
             self.lock.release()
+
+
+def finish(exit_code=None):
+    if wandb.run:
+        wandb.run.finish(exit_code=exit_code)
