@@ -25,6 +25,7 @@ Override priorities are in the reverse order of non-override settings
 import configparser
 import copy
 from datetime import datetime
+from distutils.util import strtobool
 import enum
 import getpass
 import itertools
@@ -60,7 +61,6 @@ logger = logging.getLogger("wandb")
 
 defaults: Dict[str, Union[str, int, Tuple]] = dict(
     base_url="https://api.wandb.ai",
-    show_warnings=2,
     summary_warnings=5,
     git_remote="origin",
     ignore_globs=(),
@@ -88,6 +88,7 @@ env_settings: Dict[str, Optional[str]] = dict(
     anonymous=None,
     ignore_globs=None,
     resume=None,
+    silent=None,
     root_dir="WANDB_DIR",
     run_name="WANDB_NAME",
     run_notes="WANDB_NOTES",
@@ -115,7 +116,7 @@ def _is_kaggle() -> bool:
     )
 
 
-def _error_choices(value: str, choices: List[str]) -> str:
+def _error_choices(value: str, choices: Set[str]) -> str:
     return "{} not in {}".format(value, ",".join(list(choices)))
 
 
@@ -169,6 +170,14 @@ def get_wandb_dir(root_dir: str):
     return path
 
 
+def _str_as_bool(val) -> Optional[bool]:
+    try:
+        val = bool(strtobool(val))
+    except (AttributeError, ValueError):
+        pass
+    return val if isinstance(val, bool) else None
+
+
 @enum.unique
 class SettingsConsole(enum.Enum):
     OFF = 0
@@ -206,6 +215,10 @@ class Settings(object):
     sync_symlink_latest_spec: Optional[str] = None
     settings_system_spec: Optional[str] = None
     settings_workspace_spec: Optional[str] = None
+    silent: bool = False
+    show_info: bool = True
+    show_warnings: bool = True
+    show_errors: bool = True
 
     # Private attributes
     __start_time: Optional[float]
@@ -229,9 +242,10 @@ class Settings(object):
         WORKSPACE: int = 7
         ENV: int = 8
         SETUP: int = 9
-        INIT: int = 10
-        SETTINGS: int = 11
-        ARGS: int = 12
+        LOGIN: int = 10
+        INIT: int = 11
+        SETTINGS: int = 12
+        ARGS: int = 13
 
     Console: Type[SettingsConsole] = SettingsConsole
 
@@ -252,6 +266,9 @@ class Settings(object):
         magic: Union[Dict, str, bool] = False,
         run_tags: Sequence = None,
         sweep_id=None,
+        allow_val_change: bool = None,
+        force: bool = None,
+        relogin: bool = None,
         # compatibility / error handling
         # compat_version=None,  # set to "0.8" for safer defaults for older users
         # strict=None,  # set to "on" to enforce current best practices (also "warn")
@@ -296,10 +313,12 @@ class Settings(object):
         dev_prod=None,  # in old settings files, TODO: support?
         host=None,
         username=None,
+        email=None,
         docker=None,
         _start_time=None,
         _start_datetime=None,
         _cli_only_mode=None,  # avoid running any code specific for runs
+        _disable_viewer=None,  # prevent early viewer query
         console=None,
         disabled=None,  # alias for mode=dryrun, not supported yet
         reinit=None,
@@ -307,7 +326,7 @@ class Settings(object):
         # compute environment
         show_colors=None,
         show_emoji=None,
-        show_console=None,
+        silent=None,
         show_info=None,
         show_warnings=None,
         show_errors=None,
@@ -354,6 +373,30 @@ class Settings(object):
             ret = True
         return ret
 
+    @property
+    def _silent(self) -> Optional[bool]:
+        if not self.silent:
+            return None
+        return _str_as_bool(self.silent)
+
+    @property
+    def _show_info(self) -> Optional[bool]:
+        if not self.show_info:
+            return None
+        return _str_as_bool(self.show_info)
+
+    @property
+    def _show_warnings(self) -> Optional[bool]:
+        if not self.show_warnings:
+            return None
+        return _str_as_bool(self.show_warnings)
+
+    @property
+    def _show_errors(self) -> Optional[bool]:
+        if not self.show_errors:
+            return None
+        return _str_as_bool(self.show_errors)
+
     @property
     def _noop(self) -> bool:
         return self.mode == "noop"
@@ -382,27 +425,28 @@ class Settings(object):
             if self._jupyter:
                 console = "wrap"
             elif self._windows:
-                legacy_env_var = "PYTHONLEGACYWINDOWSSTDIO"
-                if sys.version_info >= (3, 6) and legacy_env_var not in os.environ:
-                    msg = (
-                        "Set %s environment variable to enable"
-                        " proper console logging on Windows. Falling "
-                        "back to wrapping stdout/err." % legacy_env_var
-                    )
-                    wandb.termwarn(msg)
-                    logger.info(msg)
-                    console = "wrap"
-                if "tensorflow" in sys.modules:
-                    msg = (
-                        "Tensorflow detected. Stream redirection is not supported "
-                        "on Windows when tensorflow is imported. Falling back to "
-                        "wrapping stdout/err."
-                    )
-                    wandb.termlog(msg)
-                    logger.info(msg)
-                    console = "wrap"
-                else:
-                    console = "redirect"
+                console = "wrap"
+                # legacy_env_var = "PYTHONLEGACYWINDOWSSTDIO"
+                # if sys.version_info >= (3, 6) and legacy_env_var not in os.environ:
+                #     msg = (
+                #         "Set %s environment variable to enable"
+                #         " proper console logging on Windows. Falling "
+                #         "back to wrapping stdout/err." % legacy_env_var
+                #     )
+                #     wandb.termwarn(msg)
+                #     logger.info(msg)
+                #     console = "wrap"
+                # if "tensorflow" in sys.modules:
+                #     msg = (
+                #         "Tensorflow detected. Stream redirection is not supported "
+                #         "on Windows when tensorflow is imported. Falling back to "
+                #         "wrapping stdout/err."
+                #     )
+                #     wandb.termlog(msg)
+                #     logger.info(msg)
+                #     console = "wrap"
+                # else:
+                #     console = "redirect"
             else:
                 console = "redirect"
         convert: SettingsConsole = convert_dict[console]
@@ -492,6 +536,26 @@ class Settings(object):
             return
         return _error_choices(value, choices)
 
+    def _validate_silent(self, value):
+        val = _str_as_bool(value)
+        if val is None:
+            return "{} is not a boolean".format(value)
+
+    def _validate_show_info(self, value):
+        val = _str_as_bool(value)
+        if val is None:
+            return "{} is not a boolean".format(value)
+
+    def _validate_show_warnings(self, value):
+        val = _str_as_bool(value)
+        if val is None:
+            return "{} is not a boolean".format(value)
+
+    def _validate_show_errors(self, value):
+        val = _str_as_bool(value)
+        if val is None:
+            return "{} is not a boolean".format(value)
+
     def _start_run(self):
         datetime_now: datetime = datetime.now()
         time_now: float = time.time()
@@ -533,9 +597,15 @@ class Settings(object):
         self._update(env_dict, _source=self.Source.ENV)
 
     def _apply_user(self, user_settings, _logger=None):
+        _logger = _logger or logger
         _logger.info("setting user settings: {}".format(user_settings))
         self._update(user_settings, _source=self.Source.USER)
 
+    def _apply_source_login(self, login_settings, _logger=None):
+        _logger = _logger or logger
+        _logger.info("setting login settings: {}".format(login_settings))
+        self._update(login_settings, _source=self.Source.LOGIN)
+
     def _path_convert_part(self, path_part, format_dict):
         """convert slashes, expand ~ and other macros."""
 
@@ -702,16 +772,20 @@ class Settings(object):
 
     def _infer_run_settings_from_env(self):
         """Modify settings based on environment (for runs only)."""
-        # If the settings say to save code, and there's not already a program file,
-        # infer it now.
-        if self.save_code and not self.program_relpath:
-            program = _get_program()
-            if program:
-                program_relpath = _get_program_relpath_from_gitrepo(program)
-                self.update(dict(program=program, program_relpath=program_relpath))
-            else:
-                program = "<python with no main file>"
-                self.update(dict(program=program))
+        if self.disable_code:
+            self.update(dict(program="<code saving explicitly disabled>"))
+            return
+
+        # If there's not already a program file, infer it now.
+        program = self.program or _get_program()
+        if program:
+            program_relpath = self.program_relpath or _get_program_relpath_from_gitrepo(
+                program
+            )
+            self.update(dict(program=program, program_relpath=program_relpath))
+        else:
+            program = "<python with no main file>"
+            self.update(dict(program=program))
 
     def setdefaults(self, __d=None):
         __d = __d or defaults
@@ -786,7 +860,22 @@ class Settings(object):
                 d[k] = d[k].split(",")
         return d
 
-    def apply_init(self, args):
+    def _apply_login(self, args):
+        param_map = dict(key="api_key", host="base_url",)
+        args = {param_map.get(k, k): v for k, v in six.iteritems(args) if v is not None}
+        self._apply_source_login(args)
+
+    def _apply_init(self, args):
+        # prevent setting project, entity if in sweep
+        # TODO(jhr): these should be locked elements in the future
+        if self.sweep_id:
+            for key in ("project", "entity"):
+                val = args.pop(key, None)
+                if val:
+                    wandb.termwarn(
+                        "Ignored wandb.init() arg %s when running a sweep" % key
+                    )
+
         # strip out items where value is None
         param_map = dict(
             name="run_name",
@@ -808,7 +897,10 @@ class Settings(object):
                     args["resume"] = "allow"
             elif args["resume"] is True:
                 args["resume"] = "auto"
-        self.update(args)
+
+        # update settings
+        self._update(args, _source=self.Source.INIT)
+
         # handle auto resume logic
         if self.resume == "auto":
             if os.path.exists(self.resume_fname):
