@@ -95,12 +95,16 @@ def sentry_reraise(exc):
     six.reraise(type(exc), exc, sys.exc_info()[2])
 
 
-def sentry_set_scope(process_context, entity, project, url=None):
-    with configure_scope() as scope:
+def sentry_set_scope(process_context, entity, project, email=None, url=None):
+    # Using GLOBAL_HUB means these tags will persist between threads.
+    # Normally there is one hub per thread.
+    with sentry_sdk.hub.GLOBAL_HUB.configure_scope() as scope:
         scope.set_tag("process_context", process_context)
         scope.set_tag("entity", entity)
         scope.set_tag("project", project)
-        if url is not None:
+        if email:
+            scope.user = {"email": email}
+        if url:
             scope.set_tag("url", url)
 
 
@@ -214,6 +218,17 @@ MAX_SLEEP_SECONDS = 60 * 5
 VALUE_BYTES_LIMIT = 100000
 
 
+def app_url(api_url):
+    if '://api.wandb.' in api_url:
+        # cloud
+        return api_url.replace('://api.', '://')
+    elif '://api.' in api_url:
+        # onprem cloud
+        return api_url.replace('://api.', '://app.')
+    # wandb/local
+    return api_url
+
+
 def get_full_typename(o):
     """We determine types based on type names so we don't have to import
     (and therefore depend on) PyTorch, TensorFlow, etc.
@@ -874,7 +889,7 @@ def auto_project_name(program):
     # if we're in git, set project name to git repo name + relative path within repo
     root_dir = GitRepo().root_dir
     if root_dir is None:
-        return None
+        return "uncategorized"
     # On windows, GitRepo returns paths in unix style, but os.path is windows
     # style. Coerce here.
     root_dir = to_native_slash_path(root_dir)
@@ -935,6 +950,13 @@ def bytes_to_hex(bytestr):
     # Works in python2 / python3
     return codecs.getencoder('hex')(bytestr)[0].decode('ascii')
 
+def check_and_warn_old(files):
+    if 'wandb-metadata.json' in files:
+        wandb.termwarn("These runs were logged with a previous version of wandb.")
+        wandb.termwarn("Run pip install wandb<0.10.0 to get the old library and sync your runs.")
+        return True
+    return False
+
 
 class ImportMetaHook():
     def __init__(self):
